<?

 /*************************************************************************************
 * Filename: its.php
 *
 * This file is part of the ITS JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The ITS JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/its/
 *************************************************************************************/


 require_once($_path["include"]."data.php");
 require_once($_path["format"]."template.php");
 require_once($_path["plugins"]."lister/lister.php");

 class Plugin_its extends Plugin_lister
 {
  function main()
  {
   global $_benchmark, $_itsUpdateSubmitted, $_itsFileSubmitted, $_itsSubscribeSubmitted, $_config;

   if(!$this->config)
    return false;

   $this->updateError=false;
   $this->fileError=false;

   //only submit in its/item mode to ensure error is set to correct template
   if($this->config["listtype"]!="ITS"&&$this->config["listtype"]!="Item")
    return true;

   if(!$_itsUpdateSubmitted) //ensure updates are only submitted once
   {
    //update submit
    $_itsUpdateSubmitted=true;
    if($this->updateText=request("update"))
    {
     if($_benchmark) $_benchmark->start("Submit update");

     $d=Data::cache("its_updates", "add", "item");

     if($_config->userAnonymous())
     {
      $d->addField("captcha", "captcha", "form", "", 0, 5, "", "Captcha");
      $d->setRequestValue("captcha");
     }

     $d->setValue("text", $this->updateText);
     $this->updateEntry=$d->getValue("issue");

     if(!$d->submit())
      $this->updateError=$d->e->show();

     if($_benchmark) $_benchmark->stop("Submit update");
    }
   }

   if(!$_itsFileSubmitted) //ensure files are only submitted once
   {
    //file submit
    $_itsFileSubmitted=true;
    if($this->fileFile=request("file", "name", "FILES"))
    {
     $this->fileName=request("filename");
     if($_benchmark) $_benchmark->start("Submit file");

     $d=Data::cache("its_files", "add", "item");

     if($_config->userAnonymous())
     {
      $d->addField("captcha", "captcha", "form", "", 0, 5, "", "Captcha");
      $d->setRequestValue("captcha");
     }

     $d->setValue("name", $this->fileName);
     $this->fileEntry=$d->getValue("issue");

     if(!$d->submit())
      $this->fileError=$d->e->show();

     if($_benchmark) $_benchmark->stop("Submit file");
    }
   }

   if(!$_itsSubscribeSubmitted) //ensure subscribe are only submitted once
   {
    //(un)subscribe
    $_itsSubscribeSubmitted=true;
    if(request("subscribe"))
    {
     if($_benchmark) $_benchmark->start("Subscribe");

     $d=Data::cache("its_issues", "edit", "item");
     $d->clearValues();

     $d->setRequestValue("nr");
     $d->setRequestValue("subscribed");

     //submit as root to ensure issue can be editted (user only needs view rights to subscribe)
     $_config->su(true); //root
     $d->submit(); //don't bother logging errors
     $_config->su(); //original user

     if($_benchmark) $_benchmark->stop("Subscribe");
    }
   }

   return true;
  }

  function _getData()
  {
   global $_benchmark;

   if(!$this->config)
    return false;

   if($_benchmark) $_benchmark->start("Data object");

   if($this->config["listtype"]=="ITS"||$this->config["listtype"]=="Item"||$this->config["listtype"]=="Email")
   {
    $d=Data::cache("its_issues", "view", "all");
    $d->clearValues();

    //set show fields so lister->cache takes these in to account when checking cache
    $d->setShowFields("updates", "files", "its_tagsmapping");
    $d->getReferenceData("updates")->setShowFields("text");
    $d->getReferenceData("files")->setShowFields("name", "file");
    $d->getReferenceData("its_tagsmapping")->getReferenceData("its_tags")->setShowFields("tag");

    if($this->config["listcatnr"])
     $d->setValue("catnr", $this->config["listcatnr"]);

    $d->setValue("show_total", $this->config["listitems"]);

    if($this->config["pagebrowser"]) //only show "next page" if page browser is active
    {
     if(request("show_page"))
      $d->setRequestValue("show_page", request("show_page"));
     if(request("show_from"))
      $d->setRequestValue("show_from", request("show_from"));
    }

    $d->sortOrder("workflow", "reportedtime-", "its_tagsmapping"); //also ensure tags are correctly sorted
    $d->getReferenceData("its_tagsmapping")->sortOrder("its_tags");

    if($this->config["listtype"]=="Email")
    {
     if(!isset($this->emailNr))
     {
      $d->e->add("Email NR not set, please ensure Email config is called from Plugin_Module_its_issues::sendEmail", "warning", "Using latest issue as workaround");

      $d->clearValues();
      $d->setValue("show_total", 1);

      $d->get();
     }
     else
      $d->setValue("nr", $this->emailNr);

     $d->setExtra("item");
     $d->getReferenceData("updates")->sortOrder("added-");
    }
    else if(request("search")) //index search
    {
     $d->setAction("search");
     $d->setExtra("index");

     $d->setValue("search", request("search"));
     $d->sortOrder("fulltext-");

     if(!$d->getValue("show_total")) //ensure limit is set
      $d->setValue("show_total", 100);

     if(!$d->get(false, true)||!$d->getValue("show_count")) //no results, try normal search
     {
      $d->setExtra("full");

      $d->unsetValue("fulltext");
      $d->unsetValue("show_count");

      $d->setValue("search", request("search"));
      $d->sortOrder("workflow", "reportedtime-");
     }
     else //results
      $d->setValue("search", request("search")); //re-set search
    }
    else if($arg=request("p", 1))
    {
     if(preg_match("/^\d{4}-\d{2}-\d{2}$/", $arg)) //date
     {
      $arg=strtotime($arg);
      $d->setValue("reportedtime", date($d->field["reportedtime"]["format"][$d->dataformat]["datetimeformat"], $arg), 0);
      $d->setValue("reportedtime", date($d->field["reportedtime"]["format"][$d->dataformat]["datetimeformat"], $arg+86400), 1);
      $d->setOperator("reportedtime", ">=", 0);
      $d->setOperator("reportedtime", "<", 1);
      $d->setOperant("reportedtime", "AND", 0);
      $d->setOperant("reportedtime", "AND", 1);
     }
     else if(preg_match("/^\d+$/", $arg)) //issue
     {
      $d->setValue("nr", $arg);

      $this->config["listtype"]="Item";
     }
     else if(preg_match(REGEXP_ITS_ID, $arg)) //issue ID
     {
      $d->setValue("id", $arg);

      $this->config["listtype"]="Item";
     }
     else //filters
     {
      $args=request("p");
      array_shift($args); //skip first argument
      for($c=0;$c<sizeof($args);$c+=2)
      {
       if(!isset($args[$c+1])) //no secondary value argument
        break;

       if($d->isField($args[$c]))
        $d->setValue($args[$c], $args[$c+1]);
      }
     }
    }
   }
   else if($this->config["listtype"]=="Latest issues"||$this->config["listtype"]=="Latest updates") //Latest issues/updates
   {
    if($this->config["listtype"]=="Latest issues")
    {
     $d=Data::cache("its_issues", "view", "all");
     $d->sortOrder("reportedtime-");
    }
    else //updates
    {
     $d=Data::cache("its_updates", "view", "all");
     $d->sortOrder("updated-");
    }

    $d->clearValues();

    if($this->config["listcatnr"])
     $d->setValue("catnr", $this->config["listcatnr"]);

    $d->setValue("show_total", $this->config["listitems"]);
   }
   else //tags cloud
   {
    $d=Data::cache("its_tags", "view", "all");
    $d->clearValues();
    $d->sortOrder("tag+");

    if($this->config["listcatnr"])
     $d->setValue("catnr", $this->config["listcatnr"]);

    $d->field["cloud_size"]["min"]=1;
    $d->field["cloud_size"]["max"]=10;
    $d->field["cloud_size"]["type"]="nr";
   }

   if($_benchmark) $_benchmark->stop("Data object");

   return $d;
  }


  function cache(&$cacheTime, &$cacheKey)
  {
   //no caching on update/files error or search
   if($this->updateError||$this->fileError||request("search"))
    return false;

   return parent::cache($cacheTime, $cacheKey);
  }

  function _cacheTemplateUpdated(&$cacheTime, &$dc)
  {
   global $_path;

   if($this->config["listtype"]=="Item"||$this->config["listtype"]=="ITS") //item/list template
    return parent::_cacheTemplateUpdated($cacheTime, $dc); //use lister plugin
   else if($this->config["listtype"]=="Email") //Email
   {
    if($this->config["templateitem"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templateitem"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templateemail, "modified");
   }
   else if($this->config["listtype"]=="Latest issues") //Latest issues
   {
    if($this->config["templatelist"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templatelist"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templatelatestissues, "modified");
   }
   else if($this->config["listtype"]=="Latest updates") //Latest updates
   {
    if($this->config["templatelist"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templatelist"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templatelatestupdates, "modified");
   }
   else //tag cloud templates
   {
    if($this->config["templatelist"])
     $time=FS::getTimeS($_path["project"]["templates"].$this->config["templatelist"], "modified");
    else
     $time=FS::getTimeS($this->plugin->path["templates"].$this->plugin->templatecloud, "modified");
   }

   if($cacheTime<$time)
    $cacheTime=$time;

   return true;
  }



  function output()
  {
   global $_path, $_config, $_benchmark;

   if($_benchmark) $_benchmark->start("Get content");

   if(!($d=$this->_getData()))
    return false;

   $id=$d->getID();

   if($this->config["cmspage"]||($this->config["itspage"]))
   {
    $tmp=Data::cache("pager_categories", "view", "item");

    if($this->config["cmspage"])
    {
     $tmp->setValue("nr", $this->config["cmspagenr"]);
     $tmp->setShowFields("urlname");

     if($tmp->get()&&$tmp->next())
      $cmspage=$tmp->v["urlname"];
     else
      $cmspage=$this->config["cmspagenr"];
    }

    if($this->config["itspage"])
    {
     $tmp->setValue("nr", $this->config["itspagenr"]);
     $tmp->setShowFields("urlname");

     if($tmp->get()&&$tmp->next())
      $itspage=$tmp->v["urlname"];
     else
      $itspage=$this->config["itspagenr"];
    }

    unset($tmp);
   }

   $d->addField("sort_order", "text", "show", "control", false, false, implode(",", $d->sortOrder));

   $d->addField("item_url", "text", "", "control", 0);
   if($_config->get("prettyurls")) //mod rewrite
   {
    if($this->config["cmspage"])
     $d->addField("cms_url", "text", "", "control", 0, false, $_path["HTML"]["current"].$cmspage."/");
    else
     $d->addField("cms_url", "text", "", "control", 0);

    $d->addField("base_url", "text", "", "control", 0, false, $_path["HTML"]["current"]);
    $d->addField("root_url", "text", "", "control", 0, false, $_path["root"]);

    $tmp=$_path["HTML"]["current"];
    foreach($this->plugin->vars as $k => $v)
     if($k=="p"||substr($k, 0, 2)=="p[")
      $tmp.=$v."/";

    $d->addField("page_url", "text", "", "control", 0, false, $tmp);

    if($this->config["itspage"])
     $d->addField("its_url", "text", "", "control", 0, false, $_path["HTML"]["current"].$itspage."/");
    else //no specific its page, use current page
     $d->addField("its_url", "text", "", "control", 0, false, $tmp);

    unset($tmp);
   }
   else
   {
    if($this->config["cmspage"])
     $d->addField("cms_url", "text", "", "control", 0, false, $_path["HTML"]["current"].htmlentities("?=".$cmspage."&"));
    else
     $d->addField("cms_url", "text", "", "control", 0);

    if($this->config["itspage"])
     $d->addField("its_url", "text", "", "control", 0, false, $_path["HTML"]["current"].htmlentities("?=".$itspage."&"));
    else //no specific its page, use current page
     $d->addField("its_url", "text", "", "control", 0, false, htmlentities($this->plugin->url()));

    $d->addField("base_url", "text", "", "control", 0, false, "");
    $d->addField("root_url", "text", "", "control", 0, false, $_path["root"]);
    $d->addField("page_url", "text", "", "control", 0, false, htmlentities($this->plugin->url()));
   }

   //get updates+files category + rights
   if($this->config["listtype"]=="ITS"||$this->config["listtype"]=="Item")
   {
    $dtmp=Data::cache("categories", "view", "all");
    $dtmp->setShowFields("nr");
    $dtmp->clearValues();

    $dtmp->setValue("catnr", $this->config["listcatnr"]);
    $dtmp->setValue("text", "ITS UPDATES CONTAINER");
    $dtmp->setValue("show_total", 1);

    if(!$dtmp->get()||!$dtmp->next()) //can't get updates category, either no rights or setup needs to be rerun (ITS UPDATES CONTAINER in its category should exist)
    {
     $d->addField("update_catnr", "nr", "", "control", 0, false, false);
     $d->addField("update_rights", "boolean", "", "control", false, false, false);
     $d->addField("update_captcha", "boolean", "", "control", false, false, false);
    }
    else
    {
     $d->addField("update_catnr", "nr", "", "control", 0, false, $dtmp->v["nr"]);

     if($dtmp->r("add", "content"))
     {
      $d->addField("update_rights", "boolean", "", "control", false, false, true);

      if($_config->userAnonymous())
       $d->addField("update_captcha", "boolean", "", "control", false, false, true);
      else
       $d->addField("update_captcha", "boolean", "", "control", false, false, false);
     }
     else
     {
      $d->addField("update_rights", "boolean", "", "control", false, false, false);
      $d->addField("update_captcha", "boolean", "", "control", false, false, false);
     }
    }

    $dtmp=Data::cache("categories", "view", "all");
    $dtmp->setShowFields("nr");
    $dtmp->clearValues();

    $dtmp->setValue("catnr", $this->config["listcatnr"]);
    $dtmp->setValue("text", "ITS FILES CONTAINER");
    $dtmp->setValue("show_total", 1);

    if(!$dtmp->get()||!$dtmp->next()) //can't get files category, either no rights or setup needs to be rerun (ITS UPDATES CONTAINER in its category should exist)
    {
     $d->addField("file_catnr", "nr", "", "control", 0, false, false);
     $d->addField("file_rights", "boolean", "", "control", false, false, false);
     $d->addField("file_captcha", "boolean", "", "control", false, false, false);
    }
    else
    {
     $d->addField("file_catnr", "nr", "", "control", 0, false, $dtmp->v["nr"]);

     if($dtmp->r("add", "content"))
     {
      $d->addField("file_rights", "boolean", "", "control", false, false, true);

      if($_config->userAnonymous())
       $d->addField("file_captcha", "boolean", "", "control", false, false, true);
      else
       $d->addField("file_captcha", "boolean", "", "control", false, false, false);
     }
     else
     {
      $d->addField("file_rights", "boolean", "", "control", false, false, false);
      $d->addField("file_captcha", "boolean", "", "control", false, false, false);
     }
    }
   }

   //update+files submit items
   $d->addField("update_text", "text", "", "control", 0, false, "");
   $d->addField("update_error", "text", "", "control", 0, false, "");
   $d->addField("file_file", "text", "", "control", 0, false, "");
   $d->addField("file_name", "text", "", "control", 0, false, "");
   $d->addField("file_error", "text", "", "control", 0, false, "");

   //load template
   if($this->config["listtype"]=="Item") //item template
   {
    if($this->config["templateitem"])
     $templateFile=$_path["project"]["templates"].$this->config["templateitem"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templateitem;
   }
   else if($this->config["listtype"]=="ITS") //list template
   {
    if($this->config["templatelist"])
     $templateFile=$_path["project"]["templates"].$this->config["templatelist"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templatelist;
   }
   else if($this->config["listtype"]=="Email") //item template
   {
    if($this->config["templateitem"])
     $templateFile=$_path["project"]["templates"].$this->config["templateitem"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templateemail;
   }
   else if($this->config["listtype"]=="Latest issues") //Latest issues
   {
    if($this->config["templatelist"])
     $templateFile=$_path["project"]["templates"].$this->config["templatelist"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templatelatestissues;
   }
   else if($this->config["listtype"]=="Latest updates") //Latest updates
   {
    if($this->config["templatelist"])
     $templateFile=$_path["project"]["templates"].$this->config["templatelist"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templatelatestupdates;
   }
   else //tag cloud templates
   {
    if($this->config["templatelist"])
     $templateFile=$_path["project"]["templates"].$this->config["templatelist"];
    else
     $templateFile=$this->plugin->path["templates"].$this->plugin->templatecloud;
   }

   if($_benchmark) $_benchmark->start("Template");

   $template=new Format_Template(true);

   if(!$template->read($templateFile)||!$template->analyze($d))
   {
    $this->e->add($template->e);
    return false;
   }

   if($template->markerExists("_fields")&&$this->config["showfields"]) //_fields marker in template and predefined fields set
    $d->setShowFields(array_merge($this->config["showfields"], $d->check("use", "show"))); //added predefined fields to show fields
   //else //use template defined fields

   if($_benchmark) $_benchmark->stop("Template");

   if($_benchmark) $_benchmark->start("Get data");

   //get data
   if($this->config["pagebrowser"]||$template->showCount())
    $success=$d->get($template->getLevel(), true);
   else
    $success=$d->get($template->getLevel());

   if(!$success)
   {
    $this->e->add($d->e);
    return false;
   }

   //set sort_order, page_url, base_url, root_url and item_url values
   for($c=0;$c<$d->vmax;$c++)
   {
    if($this->config["listtype"]=="ITS"||$this->config["listtype"]=="Item")
    {
     $d->setDefaultValue("update_catnr", $c);
     $d->setDefaultValue("update_rights", $c);
     $d->setDefaultValue("file_catnr", $c);
     $d->setDefaultValue("file_rights", $c);
    }

    if($this->updateError&&$d->getValue("nr", $c)==$this->updateEntry)
    {
     $d->setValue("update_text", $this->updateText, $c);
     $d->setValue("update_error", $this->updateError, $c);
    }
    else
    {
     $d->setDefaultValue("update_text", $c);
     $d->setDefaultValue("update_error", $c);
    }

    if($this->fileError&&$d->getValue("nr", $c)==$this->fileEntry)
    {
     $d->setValue("file_file", $this->fileFile, $c);
     $d->setValue("file_name", $this->fileName, $c);
     $d->setValue("file_error", $this->fileError, $c);
    }
    else
    {
     $d->setDefaultValue("file_file", $c);
     $d->setDefaultValue("file_name", $c);
     $d->setDefaultValue("file_error", $c);
    }

    $d->setDefaultValue("sort_order", $c);
    $d->setDefaultValue("page_url", $c);
    $d->setDefaultValue("base_url", $c);
    $d->setDefaultValue("root_url", $c);
    $d->setDefaultValue("cms_url", $c);

    if($_config->get("prettyurls")) //mod rewrite
     $d->setValue("item_url", $d->getValue("page_url", $c).$d->getValue($id, $c)."/", $c);
    else
     $d->setValue("item_url", $d->getValue("page_url", $c).htmlentities($id."=".$d->getValue($id, $c)."&"), $c);
   }

   if($_benchmark) $_benchmark->stop("Get data");
   if($_benchmark) $_benchmark->start("Populate template");

   if($d->e->any("fatal"))
   {
    $this->e->add($d->e);
    return false;
   }

   if(!$template->populate())
   {
    $this->e->add($template->e);
    return false;
   }

   $output=$template->output();

   if($_benchmark) $_benchmark->stop("Populate template");

   if($this->config["pagebrowser"]&&$d->extra!="item")
   {
    if($_benchmark) $_benchmark->start("Page browser");

    $templatepagebrowser=new Format_Template(true);

    if($this->config["templatepagebrowser"])
     $templatepagebrowser->read($_path["project"]["templates"].$this->config["templatepagebrowser"]);

    if($templatepagebrowser->pageBrowser($d, $_path["HTML"]["url"], (request("sort")?true:false), false))
     $output.=$templatepagebrowser->output();
    else
    {
     $this->e->add($templatepagebrowser->e);
     return false;
    }

    if($_benchmark) $_benchmark->stop("Page browser");
   }

   if($_benchmark) $_benchmark->stop("Get content");

   //add Email testing error
   if($this->config["listtype"]=="Email"&&!isset($this->emailNr))
    $output=$d->e->show().$output;

   return $output;
  }

 }
?>
