<?

 /*************************************************************************************
 * Filename: modules/issues.php
 *
 * This file is part of the ITS JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The ITS JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/its/
 *************************************************************************************/


 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."module.php");
 require_once($_path["include"]."mutilate.php");
 require_once($_path["include"]."functions.php");
 require_once($_path["project"]["plugins"]."its/modules/updates.php");

 require_once($_path["format"]."email.php");
 require_once($_path["protocol"]."smtp.php");

 define("REGEXP_ITS_ID", "/^([A-Z]+)(\d{2})([A-Z]{3})(\d+)$/i");

 class Plugin_Module_its_issues extends Module
 {
  function __construct(&$data)
  {
   global $_config;

   parent::__construct($data);

   $this->data->title="ITS issues";

   $this->data->defaultFields();

   $this->data->addField("id", "text", "show;sql", "title;disabled", 0, 25, "", "ID");
   $this->data->addUnique("id", "id");

   $this->data->addField("type", "enum", "all", "", 0, false, "Issue", "");
   $this->data->enumOptions("type", "Issue", "Change", "Bug", "Question");

   $this->data->addField("summary", "text", "all", "title", 1, 255, false, "Summary");
   $this->data->addField("description", "html", "all", "wysiwyg", 0, 65536, false, "Description");


   //meta
   $this->data->addField("tags", "text", "show;form", "", 0, 255, false, "Tags");

   $this->data->addField("projectnr", "nr", "show;sql", "", 1, false, 1, "Project");
   $this->data->addReference("project", "its_projects", "nr", "*:1", "projectnr");

   $this->data->addField("plugin", "regexp", "all", "", 0, 255, false, "Plugin");
   $this->data->regexps("plugin", REGEXP_DATA_PLUGIN);

   //priority
   $this->data->addField("priority", "enum", "all", "", false, false, "", "Priority");
   $this->data->enumOptions("priority", "", "Low", "Medium", "High");

   $this->data->addField("impact", "enum", "all", "", false, false, "", "Impact");
   $this->data->enumOptions("impact", "", "Low", "Medium", "High");

   $this->data->addField("rank", "nr", "show", "", false, false, "", "Rank");


   //versioning
   $this->data->addField("version", "version", "all", "", false, false, false, "Version");
   $this->data->addField("dueversion", "version", "all", "", false, false, false, "Due in version");
   $this->data->addField("changeset", "nr", "all", "", 0, false, false, "Fixed in changeset");


   //workflow
   $this->data->addField("workflow", "nr", "all", "", 1, false, "ITS - Default", "Workflow");
   $this->data->addReference("workflow", "workflow", "nr", "*:1");


   //time/users
   $this->data->addField("reportedtime", "datetime", "sql;show", "", 1, false, "now", "Reported at");
   $this->data->addField("reportedby", "nr", "sql;show", "", 0, false, "usernr", "Reported by");
   $this->data->addReference("reportedby", $_config->get("usermodule"), "nr", "*:0");

   $this->data->addField("closedtime", "datetime", "sql;show", "disabled", false, false, "", "Closed at");
   $this->data->addField("closedby", "nr", "sql;show", "disabled", 0, false, 0, "Closed by");
   $this->data->addReference("closedby", $_config->get("usermodule"), "nr", "*:0");


   //updates
   $this->data->addReference("updates", "its_updates", "issuenr", "1:*", "nr");

   if(isset($this->data->field["updates"]))
    $this->data->field["updates"]["properties"].=";hidden";

   //files
   $this->data->addReference("files", "its_files", "issuenr", "1:*", "nr");

   if(isset($this->data->field["files"]))
    $this->data->field["files"]["properties"].=";hidden";

   //tags
   $this->data->addReference("its_tagsmapping", "its_tags", "nr", "*:*", "nr");

   if(isset($this->data->field["its_tagsmapping"]))
    $this->data->field["its_tagsmapping"]["properties"].=";hidden";

   //subscribers
   $this->data->addReference("its_subscribers", $_config->get("usermodule"), "nr", "*:*", "nr");

   if(isset($this->data->field["its_subscribers"]))
   {
    $this->data->field["its_subscribers"]["properties"].=";hidden";
    $this->data->field["its_subscribers"]["title"]="Subscribers";
   }

   $this->data->addField("subscribed", "boolean", "show;form", "", false, false, true, "Subscribed");


   $this->data->addFullTextIndex("fulltext", "summary", "description", "id", "plugin");

   $this->data->sortOrder("reportedtime");

   $this->data->group("Issue", "type", "summary", "description");
   $this->data->group("Info", "tags", "project", "plugin");
   $this->data->group("Priority", "priority", "impact");
   $this->data->group("Versioning", "version", "dueversion", "changeset");
   $this->data->group("Reporting", "reportedtime", "reportedby", "closedtime", "closedby", "its_subscribers", "subscribed", "workflow");
  }

  function viewPostGet()
  {
   if(!$this->convertTagsToText())
    return false;

   return $this->calculateRank();
  }
  function searchPostGet()
  {
   if(!$this->convertTagsToText())
    return false;

   return $this->calculateRank();
  }
  function copyPostGet(){return $this->convertTagsToText();}
  function editPostGet(){return $this->convertTagsToText();}
  function movePostGet(){return $this->convertTagsToText();}
  function deletePostGet(){return $this->convertTagsToText();}
  function ownerPostGet(){return $this->convertTagsToText();}

  function postInit(){if($this->data->action=="add"){return $this->customizeFields();} return true;}
  function viewPostGetReferences(){return $this->customizeFields();}
  function searchPostGetReferences(){return $this->customizeFields();}
  function copyPostGetReferences(){return $this->customizeFields();}
  function editPostGetReferences(){return $this->customizeFields();}
  function movePostGetReferences(){return $this->customizeFields();}
  function deletePostGetReferences(){return $this->customizeFields();}
  function ownerPostGetReferences(){return $this->customizeFields();}

  function viewPreGetErrorCheck(){return $this->convertTextToTags();}
  function addPreSubmitErrorCheck(){return $this->checkValues();}
  function copyPreSubmitErrorCheck(){return $this->checkValues();}
  function editPreSubmitErrorCheck(){return $this->checkValues();}

  function editPreSubmit(){return $this->addUpdate();}

  function viewPreGet(){return $this->showSubscribedAndRank();}
  function searchPreGet(){return $this->showSubscribedAndRank();}
  function copyPreGet(){return $this->showSubscribedAndRank();}
  function editPreGet(){return $this->showSubscribedAndRank();}
  function movePreGet(){return $this->showSubscribedAndRank();}
  function deletePreGet(){return $this->showSubscribedAndRank();}
  function ownerPreGet(){return $this->showSubscribedAndRank();}

  function addPostSubmit(){$this->sendEmail($this->data->getValue("nr")); return true;}
  function copyPostSubmit(){$this->sendEmail($this->data->getValue("nr")); return true;}
  function editPostSubmit(){$this->sendEmail($this->data->getValue("nr")); return true;}

  function showSubscribedAndRank()
  {
   //also get its_subscribers field when gathering subscribed
   if($this->data->check("use", "show", "subscribed")&&!$this->data->check("use", "show", "its_subscribers"))
    $this->data->field["its_subscribers"]["use"].=";show";

   //get priority+impact field when gathering rank
   if($this->data->check("use", "show", "rank"))
   {
    if(!$this->data->check("use", "show", "priority"))
     $this->data->field["priority"]["use"].=";show";
    if(!$this->data->check("use", "show", "impact"))
     $this->data->field["impact"]["use"].=";show";
   }

   return true;
  }


  function calculateRank()
  {
   if($this->data->check("use", "show", "rank"))
   {
    for($c=0;$c<$this->data->vmax;$c++)
    {
     $priority=array_search($this->data->getValue("priority", $c), $this->data->listOptions["priority"]);
     $impact=array_search($this->data->getValue("impact", $c), $this->data->listOptions["impact"]);

     if($impact&&$priority) //both need to be set
      $rank=$impact+(4-$priority)-1;
     else
      $rank="";

     $this->data->setValue("rank", $rank, $c);
    }
   }

   return true;
  }

  function customizeFields()
  {
   global $_config;

   //set list options for workflow
   if($this->data->getReferenceData("workflow")&&!$this->data->getReferenceData("workflow")->moduleObject->determineState($this->data))
   {
    $this->data->e->add($this->data->getReferenceData("workflow")->e);
    return false;
   }

   //set subscribed value based on its_subscribers
   if($this->data->action!="add"&&$this->data->check("use", "show", "subscribed")&&is_array($this->data->getValue("its_subscribers")))
   {
    if(in_array($_config->get("usernr"), $this->data->getValue("its_subscribers")))
     $this->data->setValue("subscribed", true);
    else
     $this->data->setValue("subscribed", false);
   }

   return true;
  }


  function checkValues()
  {
   global $_config;

   if(!$this->convertTextToTagsAndAdd())
    return false;

   //don't copy updates+files
   if($this->data->action=="copy") //copy issue
   {
    $this->data->unsetValue("updates");
    $this->data->unsetValue("files");
   }

   //generate ID
   if($this->data->action=="add"||$this->data->action=="copy"||($this->data->action=="edit"&&!$this->data->check("valuetype", "none", "project"))) //add/edit issue
   {
    //get project code
    $d=Data::cache("its_projects", "view", "item");

    $d->clearValues();
    $d->setValue("nr", $this->data->getValue("project"));

    $d->setShowFields("code");

    if(!$d->get()||!$d->next())
    {
     $this->data->e->add("Unable to generate ID, getting project code failed", "error");
     return false;
    }

    $id=$d->v["code"]; //project code
    $id.=Mutilate::changeCaseS(DT::dateConvertS($this->data->getValue("reportedtime"), $this->data->field["reportedtime"]["format"][$this->data->dataformat]["datetimeformat"], "yM"), "upper"); //year+month

    if($this->data->action=="add"||$this->data->action=="copy") //add issue
    {
     if(!($nr=$this->getUniqueID($id)))
      return false;

     $id.=Mutilate::leadingZerosS($nr, 3);
     $this->data->setValue("id", $id); //set ID
    }
    else //edit issue
    {
     //get current id
     $d=Data::cache($this->data->module, "view", "item");

     $d->clearValues();
     $d->setValue("nr", $this->data->getValue("nr"));

     $d->setShowFields("id");

     if(!$d->get()||!$d->next())
     {
      $this->data->e->add("Unable to update ID, getting current", "error");
      return false;
     }

     if(strpos($d->v["id"], $id)===0) //ID prefix the same
      $this->data->setValue("id", $d->v["id"]); //use current ID
     else #ID prefix changed
     {
      //update ID
      if(!($nr=$this->getUniqueID($id)))
       return false;

      $id.=Mutilate::leadingZerosS($nr, 3);
      $this->data->setValue("id", $id); //set ID
     }
    }
   }

   //set closed time/by when workflow is set to CLOSED
   if(!$this->data->check("valuetype", "none", "workflow")&&$this->data->getReferenceData("workflow")->moduleObject->getStateCode($this->data->getValue("workflow"))=="CLOSED") //workflow set to closed
   {
    $setClosed=false;

    if($this->data->action=="add"||$this->data->action=="copy") //new issue, just set values
     $setClosed=true;
    else if($this->data->action=="edit") //edit, check if issue isn't closed already
    {
     $d=Data::cache($this->data->module, "view", "item");
     $d->clearValues();
     $d->setValue("nr", $this->data->getValue("nr"));

     $d->setShowFields("workflow");

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }

     if($d->getValue("workflow")!=$this->data->getValue("workflow")) //issue not closed yet
      $setClosed=true;
    }

    if($setClosed) //set closed values
    {
     if(!$this->data->getValue("closedtime"))
      $this->data->setValue("closedtime", date($this->data->field["closedtime"]["format"][$this->data->dataformat]["datetimeformat"]));

     $this->data->setValue("closedby", $_config->get("usernr"));
    }
    else //make sure values aren't set
    {
     $this->data->unsetValue("closedtime");
     $this->data->unsetValue("closedby");
    }
   }

   //(un)subscribe current user
   if(!$this->data->check("valuetype", "none", "subscribed"))
   {
    if($this->data->action!="add"&&$this->data->check("valuetype", "none", "its_subscribers")) //subscribers not gathered
    {
     $d=Data::cache("its_issues", "view", "item"); //view item (to ensure same rights apply)
     $d->setShowFields("its_subscribers");
     $d->clearValues();

     $d->setValue("nr", $this->data->getValue("nr"));

     if(!$d->get()||!$d->next())
     {
      $this->data->e->add("Unable to change subscribers, getting subscribers failed", "error");
      return false;
     }

     $this->data->setValue("its_subscribers", $d->v["its_subscribers"]); //set original subscribers list
    }
    //else new subscribers list already submitted

    //update subscribers list
    $subscribers=$this->data->getValue("its_subscribers");

    if($this->data->getValue("subscribed")&&!in_array($_config->get("usernr"), $subscribers)) //user subscribe but not in subscribers list
     $subscribers[]=$_config->get("usernr");
    else if(!$this->data->getValue("subscribed")&&in_array($_config->get("usernr"), $subscribers)) //user unsubscribe and in subscribers list
     $subscribers=array_remove($subscribers, $_config->get("usernr"));
    else //already (un)subscribed
     $subscribers=false;

    if($subscribers!==false) //subscribers list changed
     $this->data->setValue("its_subscribers", $subscribers);
   }

   return true;
  }


  function getUniqueID($idPrefix)
  {
   //get highest id
   $d=Data::cache($this->data->module, "view", "all");

   $d->clearValues();
   $d->setValue("show_total", 1);
   $d->setValue("id", $idPrefix);
   $d->setOperator("id", "start like");

   $d->sortOrder("id-");
   $d->setShowFields("id");

   if(!$d->get())
   {
    $this->data->e->add("Unable to generate ID, getting highest failed", "error");
    $this->data->e->add($d->e);
    return false;
   }

   if(!$d->next()) //no IDs yet
    return 1;

   //ID found
   if(!preg_match(REGEXP_ITS_ID, $d->v["id"], $m))
   {
    $this->data->e->add("Unable to generate ID, highest is invalid", "error");
    return false;
   }

   return $m[4]+1;
  }


  /* Tags */

  function convertTagsToText()
  {
   if($this->data->check("use", "show", "tags"))
   {
    $d=&$this->data->getReferenceData("its_tagsmapping"); //get its_tagsmapping

    //setup its_tags reference
    $dt=&$d->getReferenceData("its_tags");
    $showFields=$dt->check("use", "show"); //store show fields
    $dt->setShowFields("tag");

    for($c=0;$c<$this->data->vmax;$c++)
    {
     $d->setAction("view");
     $d->setExtra("all");
     $d->clearValues();
     $d->setValue($this->data->module, $this->data->getValue("nr", $c));

     $tags=Array();
     if($d->get(1))
     {
      while($d->next())
       $tags[]=$d->v["its_tags"]["tag"];
     }

     $this->data->setValue("tags", implode(", ", $tags), $c);
    }

    $dt->setShowFields($showFields); //restore show fields
   }

   return true;
  }

  function convertTextToTags()
  {
   $tagsmapping=0;
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(($text=$this->data->getValue("tags", $c))&&preg_match_all("/([^,]+),?\s*/", $text, $m)) //match comma separated list of tags
    {
     $text=$m[1];
     $tags=Array();

     if(!isset($d)) //init tags data object
     {
      $d=Data::cache("its_tags", "view", "all");
      $d->setShowFields("nr");
     }

     $x=0;
     $d->clearValues();
     foreach($text as $tag) //find correspondig tag nr
     {
      $d->setValue("tag", $tag, $x);
      $d->setOperant("tag", "OR", $x);
      $x++;
     }

     if(!$d->get())
     {
      $this->data->e->add($d->e);
      return false;
     }
     else
     {
      if($d->vmax==0) //tag(s) not found
      {
       $this->data->setValue("its_tagsmapping", 0, $tagsmapping); //ensure no results are return (unless other tags are found)
       $tagsmapping++;
      }
      else
      {
       while($d->next())
       {
        if($tagsmapping>0)
         $this->data->setOperant("its_tagsmapping", "OR", $tagsmapping-1);

        $this->data->setValue("its_tagsmapping", $d->v["nr"], $tagsmapping);
        $tagsmapping++;
       }
      }
     }
    }
   }

   return true;
  }


  function convertTextToTagsAndAdd()
  {
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(($text=$this->data->getValue("tags", $c))&&preg_match_all("/([^,]+),?\s*/", $text, $m)) //match comma separated list of tags
    {
     $text=$m[1];
     $tags=Array();

     if(!isset($d)) //init tags data object
     {
      //get catnr for tags in edit mode
      if($this->data->action=="edit"&&$this->data->getValueType("catnr")=="default")
      {
       $d=Data::cache($this->data->module, "view", "item");
       $d->setValue("nr", $this->data->getValue("nr"));
       $d->setShowFields("catnr");

       if($d->get()&&$d->next())
        $this->data->setValue("catnr", $d->v["catnr"]);

       unset($d);
      }

      $d=Data::cache("its_tags", "view", "all");
      $d->setShowFields("nr");
     }

     foreach($text as $tag) //find correspondig tag nr
     {
      $d->clearValues();
      $d->setValue("tag", $tag);

      if($d->get()&&$d->vmax) //tag exists
       $tags[]=$d->getValue("nr"); //store tag+id
      else
      {
       //add tag to database
       $d->setAction("add");
       $d->setExtra("item");

       $d->clearValues();
       $d->setValue("tag", $tag);
       $d->setValue("catnr", $this->data->getValue("catnr"));

       if(!$d->submit())
       {
        $this->data->e->add($d->e);
        return false;
       }

       $tags[]=$d->getValue("nr"); //store tag+id

       //reset data object
       $d->setAction("view");
       $d->setExtra("all");
      }
     }

     $this->data->setValue("its_tagsmapping", $tags, $c);
    }
   }

   return true;
  }


  function addUpdate()
  {
   $fields=Array("id", "type", "summary", "description", "project", "plugin", "tags", "priority", "impact", "workflow", "version", "dueversion", "changeset");

   if(!array_intersect($this->data->check("valuetype", "set"), $fields)) //no fields in field list updated
    return true; //no update

   //remove project/workflow from fields list if no values are set
   if($this->data->check("valuetype", "none", "project"))
    $fields=array_remove($fields, "project");
   if($this->data->check("valuetype", "none", "workflow"))
    $fields=array_remove($fields, "workflow");

   //get old values
   $d=clone $this->data;
   $d->setShowFields($fields);

   if(!$d->get())
    return false;

   //get projects
   if(in_array("project", $fields))
   {
    $projects=Array();
    for($c=0;$c<$this->data->vmax;$c++)
    {
     if(!in_array($d->getValue("project", $c), $projects))
      $projects[]=$d->getValue("project", $c);
     if(!in_array($this->data->getValue("project", $c), $projects))
      $projects[]=$this->data->getValue("project", $c);
    }

    $d2=Data::cache("its_projects", "view", "item");
    $d2->setShowFields("name");

    for($c=0;$c<sizeof($projects);$c++)
     $d2->setValue("nr", $projects[$c], $c);

    if(!$d2->get())
    {
     $this->data->e->add("Unable to add issue update", "error", "Can't get projects");
     $this->data->e->add($d2->e);
     return false;
    }

    $projects=Array();
    while($d2->next())
     $projects[$d2->v["nr"]]=$d2->v["name"];
   }


   //get workflows
   if(in_array("workflow", $fields))
   {
    $workflows=Array();
    for($c=0;$c<$this->data->vmax;$c++)
    {
     if(!in_array($d->getValue("workflow", $c), $workflows))
      $workflows[]=$d->getValue("workflow", $c);
     if(!in_array($this->data->getValue("workflow", $c), $workflows))
      $workflows[]=$this->data->getValue("workflow", $c);
    }

    $d2=Data::cache("workflow", "view", "item");
    $d2->setShowFields("text");

    for($c=0;$c<sizeof($workflows);$c++)
     $d2->setValue("nr", $workflows[$c], $c);

    if(!$d2->get())
    {
     $this->data->e->add("Unable to add issue update", "error", "Can't get workflows");
     $this->data->e->add($d2->e);
     return false;
    }

    $workflows=Array();
    while($d2->next())
     $workflows[$d2->v["nr"]]=$d2->v["text"];
   }

   //compare fields
   for($c=0;$c<$this->data->vmax;$c++)
   {
    $text="Issue updated<br/>";

    foreach($fields as $field)
    {
     if(!$this->data->check("valuetype", "none", $field)&&$d->getValue($field, $c)!=$this->data->getValue($field, $c))
     {
      if($field=="description")
       $text.=$this->data->title($field).": ".Mutilate::stripS(Mutilate::stripTagsS($d->getValue($field, $c)))." => ".Mutilate::stripS(Mutilate::stripTagsS($this->data->getValue($field, $c)))."<br/>";
      else if($field=="project")
       $text.=$this->data->title($field).": ".$projects[$d->getValue($field, $c)]." => ".$projects[$this->data->getValue($field, $c)]."<br/>";
      else if($field=="workflow")
       $text.=$this->data->title($field).": ".$workflows[$d->getValue($field, $c)]." => ".$workflows[$this->data->getValue($field, $c)]."<br/>";
      else
       $text.=$this->data->title($field).": ".$d->getValue($field, $c)." => ".$this->data->getValue($field, $c)."<br/>";
     }
    }

    if(!Plugin_Module_its_updates::add($this->data->getValue("nr", $c), $text, false)) //don't send email, done in editPostSubmit
    {
     $this->data->e->add("Unable to add issue update", "error");
     return false;
    }
   }

   $this->data->unsetValue("updates"); //don't modify updates to ensure new updates are in the set
   return true;
  }


  function sendEmail($nr)
  {
   global $_config, $_path;

   //get user/email fields
   $d=clone $this->data;

   if(!($emailFields=$d->getReferenceData("its_subscribers")->getReferenceData($_config->get("usermodule"))->check("type", "email"))) //no email field in usermodule
    return true; //don't send email

   //load its Email config
   $p=new Plugin("its");

   if(!$p->info()) //load info
    return true; //don't send email

   if(!$p->config("Email")) //no email configuration
    return true; //don't send email

   //init plugin frontend
   if(!$p->initClass("frontend")||!($lister=$p->getClass("frontend")))
   {
    $this->data->e->add($p->e);
    return false;
   }

   $lister->emailNr=(int)$nr; //set nr to force (email) item page
   $lister->updateError=false;
   $lister->fileError=false;

   //generate message
   if(!($message=$lister->output()))
   {
    $this->data->e->add($lister->e);
    return false;
   }

   //get current URL
   $url=(isset($_SERVER["HTTPS"])?"https":"http")."://".$_SERVER["HTTP_HOST"];

   //fix root links
   $message=preg_replace("!(<[^>]+(href|src)=)(['\"])(/[^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\3".$url."\\4\\5\\6", $message);

   //fix non root links (don't fix paths that start with a / or contain a : (http: / mailto:))
   $message=preg_replace("!(<[^>]+(href|src)=)(['\"])([^/][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\3".$url.$_path["HTML"]["current"]."\\4\\5\\6", $message);

   //get issue info and subscribers' email addresses
   $d->setAction("view");
   $d->setExtra("item");

   $d->clearValues();
   $d->setValue("nr", $nr);

   $d->setShowFields("workflow", "type", "id", "its_subscribers");
   $d->getReferenceData("workflow")->setShowFields("text");
   $d->getReferenceData("its_subscribers")->getReferenceData($_config->get("usermodule"))->setShowFields($emailFields);

   if(!$d->get(1)||!$d->next())
   {
    $this->e->add($d->e);
    return false;
   }

   $subject=$d->v["workflow"]["text"]." ".$d->v["type"].": ".$d->v["id"];

   $to=Array();
   foreach($d->v["its_subscribers"] as $subscriber)
    foreach($emailFields as $field)
     if(!in_array($subscriber[$field], $to))
      $to[]=$subscriber[$field];

   //prepare email
   $email=new Format_Email();
   $email->from("its@".$email->config["defaultDomain"]);

   foreach($to as $address)
    $email->to($address);

   $email->subject($subject);
   $email->message($message, true); //html+text message

   //send email
   $smtp=new Protocol_SMTP();
   if(!$smtp->send($email))
   {
    $this->data->e->add($smtp->e);
    return false;
   }

   return true;
  }

 }
?>
